package fr.itsonus.bousoleplussbackend.usecase;

import fr.itsonus.bousoleplussbackend.domain.auth.AuthenticationService;
import fr.itsonus.bousoleplussbackend.domain.quiz.model.Quiz;
import fr.itsonus.bousoleplussbackend.domain.quiz.model.QuizDetailed;
import fr.itsonus.bousoleplussbackend.infrastructure.postgres.repositories.QuizCacheProxyRepository;
import fr.itsonus.bousoleplussbackend.infrastructure.postgres.repositories.ResponseCacheProxyRepository;
import fr.itsonus.bousoleplussbackend.payload.request.QuizRequest;
import jakarta.transaction.Transactional;
import jakarta.validation.Valid;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.rest.webmvc.ResourceNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestBody;

@Slf4j
@Service
@AllArgsConstructor
public class QuizUseCase {

    private final QuizCacheProxyRepository quizRepository;
    private final ResponseCacheProxyRepository responseRepository;
    private final BundleUseCase bundleUseCase;
    private final AuthenticationService authenticationService;

    @Transactional
    public Quiz create(@Valid @RequestBody QuizRequest request) {
        var quiz = quizRepository.createNew(request.bundleId(), authenticationService.getCurrentUser());
        responseRepository.saveQuizzResponses(request.bundleId(), quiz, request.responses());
        return quiz;
    }

    public Page<QuizDetailed> findAllForCurrentUser(Long bundleId, Pageable pageable) {
        if (!bundleUseCase.existsForCurrentUser(bundleId)) {
            throw new ResourceNotFoundException("Bundle not found");
        }
        return quizRepository.findAllByBundleId(bundleId, pageable)
                .map(postgresQuizDto -> QuizDetailed.from(postgresQuizDto, responseRepository.findAllByQuizId(postgresQuizDto.getId())));
    }

    public QuizDetailed findById(Long id) {
        var currentUser = authenticationService.getCurrentUser();
        return quizRepository.findByIdAndUserId(id, currentUser.getId())
                .map(quizz -> QuizDetailed.from(quizz, responseRepository.findAllByQuizId(quizz.getId())))
                .orElseThrow(() -> new ResourceNotFoundException("Quiz with id " + id + " not found"));
    }
}
